import { __read, __spread, __values } from "tslib";
import { isObservable, Observable, of, Subject } from 'rxjs';
import { switchMap, takeUntil } from 'rxjs/operators';
import { createDestroyer } from './destroyer';
/**
 * Operator to split a stream of Objects into it's individual property streams
 *
 *
 * Source:
 * ---{ a:1, b:42 }-----{ a: 2, c: 1 }----{ b: 1337 }-------|
 *
 *
 * Result:
 * ---key:a,key:b-------key:c-------------key:b-------------|
 *
 *
 * Where:
 *
 *    key:a
 *    1-----------------2-----------------------------------|
 *
 *    key:b                               key:b
 *    42----------------|                 42----------------|
 *
 *                      key:c
 *                      1-----------------------------------|
 */
export function splitPropsToStreams() {
    return function (source$) {
        return new Observable(function (observer) {
            var propSubjectRegistry = new Map();
            var subscription = source$.subscribe({
                next: function (change) {
                    var e_1, _a, e_2, _b;
                    var changeEntries = Object.entries(change);
                    try {
                        for (var changeEntries_1 = __values(changeEntries), changeEntries_1_1 = changeEntries_1.next(); !changeEntries_1_1.done; changeEntries_1_1 = changeEntries_1.next()) {
                            var _c = __read(changeEntries_1_1.value, 2), key = _c[0], value = _c[1];
                            var source$_1 = void 0;
                            if (!propSubjectRegistry.has(key)) {
                                source$_1 = new Subject();
                                var _d = __read(createDestroyer(), 2), destroy = _d[0], destroy$ = _d[1];
                                propSubjectRegistry.set(key, { destroy: destroy, source$: source$_1 });
                                var propStream = source$_1.pipe(switchMap(function (o) { return o; }), takeUntil(destroy$));
                                propStream['key'] = key;
                                observer.next(propStream);
                            }
                            if (!source$_1) {
                                source$_1 = propSubjectRegistry.get(key).source$;
                            }
                            source$_1.next(isObservable(value) ? value : of(value));
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (changeEntries_1_1 && !changeEntries_1_1.done && (_a = changeEntries_1.return)) _a.call(changeEntries_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    try {
                        for (var _e = __values(propSubjectRegistry.keys()), _f = _e.next(); !_f.done; _f = _e.next()) {
                            var oldKey = _f.value;
                            if (oldKey in change) {
                                continue;
                            }
                            var _g = propSubjectRegistry.get(oldKey), destroy = _g.destroy, source$_2 = _g.source$;
                            source$_2.complete();
                            destroy();
                            propSubjectRegistry.delete(oldKey);
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                },
                error: function (e) {
                    observer.error(e);
                },
                complete: function () {
                    observer.complete();
                },
            });
            // complete all streams on unsubscription
            subscription.add(function () {
                __spread(propSubjectRegistry.values()).forEach(function (_a) {
                    var destroy = _a.destroy, source$ = _a.source$;
                    source$.complete();
                    destroy();
                });
            });
            return subscription;
        });
    };
}
